{% extends 'oauthost/base.html' %}{% load i18n %}
{% block oauthost_contents %}
<form method="post" action="{{ action_uri }}">
    <p>{% blocktrans %}To return to the application please push the button below.{% endblocktrans %}</p>
    <div id="box_buttons"><input type="submit" name="confirmed" value="{% trans "Return" %}"></div>
</form>
{% endblock %}