{% extends 'oauthost/base.html' %}{% load i18n %}
{% block oauthost_contents %}
    <p>{% blocktrans %}Access to this resource is restricted. Please provide appropriate credentials within the request to proceed.{% endblocktrans %}</p>
{% endblock %}