{% extends 'oauthost/base.html' %}{% load i18n %}
{% block oauthost_contents %}
    <p>{% blocktrans %}Sorry, but you have insufficient privileges to access this resource.{% endblocktrans %}</p>
{% endblock %}